@@ -21,28 +21,23 @@
 struct options opt;
 
 const struct option_info options_info[] = {
-  { C_R, OPT_SETUIDGID,   'u',  NULL,       required_argument,
-    "set uid, gid and supplementary groups", "[:]USER[:GROUP]*", },
-  { C_R, OPT_ENVUIDGID,   'U',  NULL,       required_argument,
-    "set UID and GID vars", "[:]USER[:GROUP]", },
-  { C_R, OPT_ARGV0,       'b',  NULL,       required_argument,
-    "launch program with ARGV0 as the argv[0]", "ARGV0" },
-  { C_R, OPT_ENVDIR,      'e',  NULL,       required_argument,
-    "populate environment from directory", "DIR" },
+  { C_R, OPT_SETUIDGID,   'u',  NULL,       required_argument, "set uid, gid and supplementary groups", "[:]USER[:GROUP]*", },
+  { C_R, OPT_ENVUIDGID,   'U',  NULL,       required_argument, "set UID and GID vars", "[:]USER[:GROUP]", },
+  { C_R, OPT_ARGV0,       'b',  NULL,       required_argument, "launch program with ARGV0 as the argv[0]", "ARGV0" },
+  { C_R, OPT_ENVDIR,      'e',  NULL,       required_argument, "populate environment from directory", "DIR" },
   { C_R, OPT_CHROOT,      '/',  NULL,       required_argument, "change root directory", "DIR" },
   { C_R, OPT_CHDIR,       'C',  NULL,       required_argument, "change directory", "DIR" },
   { C_R, OPT_NICE,        'n',  NULL,       required_argument, "adjust niceness", "INC" },
   { C_R, OPT_LOCK_WAIT,   'l',  NULL,       required_argument, "wait for lock", "FILE" },
   { C_R, OPT_LOCK,        'L',  NULL,       required_argument, "obtain lock; fail fast", "FILE" },
-  { C_L, OPT_LOCKOPT_WAIT,'N',  NULL,       no_argument,       "wait for lock (default)" },
-  { C_L, OPT_LOCKOPT_TRY, 'n',  NULL,       no_argument,       "don't wait for lock" },
-  { C_L, OPT_LOCKOPT_NOISY,'X', NULL,       no_argument,       "fail noisily (default)" },
-  { C_L, OPT_LOCKOPT_QUIET,'x', NULL,       no_argument,       "fail silently" },
-  { C_RS,OPT_LIMIT_MEM,   'm',  NULL,       required_argument,
-    "set soft DATA, STACK, MEMLOCK and AS limits", "BYTES" },
+  { C_L, OPT_LOCKOPT_WAIT,'N',  NULL,       no_argument,       "wait for lock (default)", NULL },
+  { C_L, OPT_LOCKOPT_TRY, 'n',  NULL,       no_argument,       "don't wait for lock", NULL },
+  { C_L, OPT_LOCKOPT_NOISY,'X', NULL,       no_argument,       "fail noisily (default)", NULL },
+  { C_L, OPT_LOCKOPT_QUIET,'x', NULL,       no_argument,       "fail silently", NULL },
+  { C_RS,OPT_LIMIT_MEM,   'm',  NULL,       required_argument, "set soft DATA, STACK, MEMLOCK and AS limits", "BYTES" },
   { C_RS,OPT_RLIMIT_DATA, 'd',  NULL,       required_argument, "set RLIMIT_DATA", "BYTES" },
   { C_XS,OPT_RLIMIT_STACK,'s',  NULL,       required_argument, "set RLIMIT_STACK", "BYTES" },
-  { C_S, OPT_RLIMIT_MEMLOCK,'l', NULL,      required_argument, "set RLIMIT_MEMLOCK", "BYTES" },
+  { C_S, OPT_RLIMIT_MEMLOCK,'l',NULL,       required_argument, "set RLIMIT_MEMLOCK", "BYTES" },
   { C_XS,OPT_RLIMIT_AS,   'a',  NULL,       required_argument, "set RLIMIT_AS", "BYTES" },
   { C_RS,OPT_RLIMIT_NOFILE,'o', NULL,       required_argument, "set RLIMIT_NOFILE", "FILES" },
   { C_RS,OPT_RLIMIT_NPROC,'p',  NULL,       required_argument, "set RLIMIT_NPROC", "PROCS" },
@@ -50,59 +45,54 @@ const struct option_info options_info[] = {
   { C_RS,OPT_RLIMIT_CORE, 'c',  NULL,       required_argument, "set RLIMIT_CORE", "BYTES" },
   { C_XS,OPT_RLIMIT_RSS,  'r',  NULL,       required_argument, "set RLIMIT_RSS", "BYTES" },
   { C_RS,OPT_RLIMIT_CPU,  't',  NULL,       required_argument, "set RLIMIT_CPU", "SECONDS" },
-  { C_X, OPT_RLIMIT_MEMLOCK,'\0',"limit-memlock", required_argument, "set RLIMIT_MEMLOCK", "BYTES" },
-  { C_X, OPT_RLIMIT_MSGQUEUE, '\0', "limit-msgqueue", required_argument, "set RLIMIT_MSGQUEUE", "BYTES" },
-  { C_X, OPT_RLIMIT_NICE, '\0', "limit-nice", required_argument, "set RLIMIT_NICE", "NICENESS" },
-  { C_X, OPT_RLIMIT_RTPRIO, '\0', "limit-rtprio", required_argument, "set RLIMIT_RTPRIO", "PRIO" },
-  { C_X, OPT_RLIMIT_RTTIME, '\0', "limit-rttime", required_argument, "set RLIMIT_RTTIME", "MS" },
-  { C_X, OPT_RLIMIT_SIGPENDING, '\0', "limit-sigpending", required_argument, "set RLIMIT_SIGPENDING", "NUM" },
-  { C_ALL,OPT_VERBOSE,    'v',  "verbose",  no_argument,       "be verbose" },
-  { C_R, OPT_VERSION,     'V',  "version",  no_argument,       "show " NAME_STR " version" },
-  { C_R, OPT_PGRPHACK,    'P',  NULL,       no_argument,       "run in new process group" },
-  { C_R, OPT_CLOSE_STDIN, '0',  NULL,       no_argument,       "close stdin" },
-  { C_R, OPT_CLOSE_STDOUT,'1',  NULL,       no_argument,       "close stdout" },
-  { C_R, OPT_CLOSE_STDERR,'2',  NULL,       no_argument,       "close stderr" },
-  { C_X, OPT_LEGACY,      '@',  NULL,       no_argument,
-    "restricts following options to chpst(8) ones" },
-  { C_X, OPT_HELP,        'h',  "help",     no_argument,       "show help" },
+  { C_X, OPT_RLIMIT_MEMLOCK,   '\0', "limit-memlock",   required_argument, "set RLIMIT_MEMLOCK", "BYTES" },
+  { C_X, OPT_RLIMIT_MSGQUEUE,  '\0', "limit-msgqueue",  required_argument, "set RLIMIT_MSGQUEUE", "BYTES" },
+  { C_X, OPT_RLIMIT_NICE,      '\0', "limit-nice",      required_argument, "set RLIMIT_NICE", "NICENESS" },
+  { C_X, OPT_RLIMIT_RTPRIO,    '\0', "limit-rtprio",    required_argument, "set RLIMIT_RTPRIO", "PRIO" },
+  { C_X, OPT_RLIMIT_RTTIME,    '\0', "limit-rttime",    required_argument, "set RLIMIT_RTTIME", "MS" },
+  { C_X, OPT_RLIMIT_SIGPENDING,'\0', "limit-sigpending",required_argument, "set RLIMIT_SIGPENDING", "NUM" },
+  { C_X, OPT_RLIMIT_LOCKS,     '\0', "limit-locks",     required_argument, "set RLIMIT_LOCKS", "NUM" },
+  { C_ALL,OPT_VERBOSE,    'v',  "verbose",  no_argument,       "be verbose", NULL },
+  { C_R, OPT_VERSION,     'V',  "version",  no_argument,       "show " NAME_STR " version", NULL },
+  { C_R, OPT_PGRPHACK,    'P',  NULL,       no_argument,       "run in new process group", NULL },
+  { C_R, OPT_CLOSE_STDIN, '0',  NULL,       no_argument,       "close stdin", NULL },
+  { C_R, OPT_CLOSE_STDOUT,'1',  NULL,       no_argument,       "close stdout", NULL },
+  { C_R, OPT_CLOSE_STDERR,'2',  NULL,       no_argument,       "close stderr", NULL },
+  { C_X, OPT_LEGACY,      '@',  NULL,       no_argument,       "restricts following options to chpst(8) ones", NULL },
+  { C_X, OPT_HELP,        'h',  "help",     no_argument,       "show help", NULL },
   { C_X, OPT_FILE,        '\0', "file",     required_argument, "read options from file", "FILE" },
-  { C_X, OPT_EXIT,        '\0', "exit",     optional_argument,
-    "exit (with optional RETCODE)", "RETCODE" },
-  { C_X, OPT_MOUNT_NS,    '\0', "mount-ns", no_argument,       "create mount namespace" },
-  { C_X, OPT_NET_NS,      '\0', "net-ns",   no_argument,       "create net namespace" },
-  { C_X, OPT_USER_NS,     '\0', "user-ns",  no_argument,       "create user namespace" },
-  { C_X, OPT_PID_NS,      '\0', "pid-ns",   no_argument,       "create pid namespace" },
-  { C_X, OPT_UTS_NS,      '\0', "uts-ns",   no_argument,       "create uts namespace" },
-  { C_X, OPT_NET_ADOPT,   '\0', "adopt-net",required_argument,
-    "adopt net namespace", "NS-PATH" },
-  { C_X, OPT_PRIVATE_RUN, '\0', "private-run",no_argument,     "create private /run" },
-  { C_X, OPT_PRIVATE_TMP, '\0', "private-tmp",no_argument,     "create private /tmp" },
-  { C_X, OPT_PROTECT_HOME,'\0', "protect-home",no_argument,    "protect home directories" },
-  { C_X, OPT_RO_SYS,      '\0', "ro-sys",     no_argument,     "create read only system" },
-  { C_X, OPT_CAPBS_KEEP,  '\0', "cap-bs-keep",required_argument,
-    "restrict capabilities bounding set", "CAP[,...]" },
-  { C_X, OPT_CAPBS_DROP,  '\0', "cap-bs-drop",required_argument,
-    "drop from capabilities bounding set", "CAP[,...]" },
-  { C_X, OPT_CAPS_KEEP,   '\0', "caps-keep", required_argument,
-    "keep (only) these capabilities", "CAP[,...]" },
-  { C_X, OPT_CAPS_DROP,   '\0', "caps-drop", required_argument,
-    "drop these capabilities", "CAP[,...]" },
-  { C_X, OPT_FORK_JOIN,   '\0', "fork-join", no_argument,      "fork and wait for process" },
-  { C_X, OPT_NEW_ROOT,    '\0', "new-root",  no_argument,      "create a new root fs" },
-  { C_X, OPT_NO_NEW_PRIVS,'\0', "no-new-privs", no_argument,   "no new privileges" },
-  { C_X, OPT_CPUS,        '\0', "cpus",      required_argument,"set CPU affinity", "AFFINITY" },
-  { C_X, OPT_CPU_SCHED,   '\0', "cpu-scheduler", required_argument,
-    "set CPU scheduler policy", "POLICY" },
-  { C_X, OPT_IO_SCHED,    '\0', "io-scheduler",   required_argument,
-    "set I/O scheduling class", "rt|best-effort|idle[:PRIORITY]"},
+  { C_X, OPT_EXIT,        '\0', "exit",     optional_argument, "exit (with optional RETCODE)", "RETCODE" },
+  { C_X, OPT_MOUNT_NS,    '\0', "mount-ns", no_argument,       "create mount namespace", NULL },
+  { C_X, OPT_NET_NS,      '\0', "net-ns",   no_argument,       "create net namespace", NULL },
+  { C_X, OPT_USER_NS,     '\0', "user-ns",  no_argument,       "create user namespace", NULL },
+  { C_X, OPT_PID_NS,      '\0', "pid-ns",   no_argument,       "create pid namespace", NULL },
+  { C_X, OPT_UTS_NS,      '\0', "uts-ns",   no_argument,       "create uts namespace", NULL },
+  { C_X, OPT_NET_ADOPT,   '\0', "adopt-net",required_argument, "adopt net namespace", "NS-PATH" },
+  { C_X, OPT_PRIVATE_RUN, '\0', "private-run",  no_argument,    "create private /run", NULL },
+  { C_X, OPT_PRIVATE_TMP, '\0', "private-tmp",  no_argument,    "create private /tmp", NULL },
+  { C_X, OPT_PROTECT_HOME,'\0', "protect-home", no_argument,    "hide home directories", NULL },
+  { C_X, OPT_RO_HOME,     '\0', "ro-home",      no_argument,    "make home directories read only", NULL },
+  { C_X, OPT_RO_SYS,      '\0', "ro-sys",       no_argument,    "create read only system", NULL },
+  { C_X, OPT_RO_ETC,      '\0', "ro-etc",       no_argument,    "make config files read only", NULL },
+  { C_X, OPT_CAPBS_KEEP,  '\0', "cap-bs-keep",  required_argument, "restrict capabilities bounding set", "CAP[,...]" },
+  { C_X, OPT_CAPBS_DROP,  '\0', "cap-bs-drop",  required_argument, "drop from capabilities bounding set", "CAP[,...]" },
+  { C_X, OPT_CAPS_KEEP,   '\0', "caps-keep",    required_argument, "keep (only) these capabilities", "CAP[,...]" },
+  { C_X, OPT_CAPS_DROP,   '\0', "caps-drop",    required_argument, "drop these capabilities", "CAP[,...]" },
+  { C_X, OPT_FORK_JOIN,   '\0', "fork-join",    no_argument,   "fork and wait for process", NULL },
+  { C_X, OPT_NEW_ROOT,    '\0', "new-root",     no_argument,   "create a new root fs", NULL },
+  { C_X, OPT_NO_NEW_PRIVS,'\0', "no-new-privs", no_argument,   "no new privileges", NULL },
+  { C_X, OPT_CPUS,        '\0', "cpus",         required_argument, "set CPU affinity", "AFFINITY" },
+  { C_X, OPT_CPU_SCHED,   '\0', "cpu-scheduler",required_argument, "set CPU scheduler policy", "POLICY" },
+  { C_X, OPT_IO_SCHED,    '\0', "io-scheduler", required_argument, "set I/O scheduling class", "rt|best-effort|idle[:PRIORITY]"},
   { C_X, OPT_UMASK,       '\0', "umask",     required_argument,"set umask", "MODE" },
   { C_X, OPT_APP,         '\0', "app",       required_argument,"define application name", "NAME"},
-  { C_X, OPT_RUN_DIR,     '\0', "run-dir",   no_argument,      "create run dir" },
-  { C_X, OPT_STATE_DIR,   '\0', "state-dir", no_argument,      "create state dir" },
-  { C_X, OPT_CACHE_DIR,   '\0', "cache-dir", no_argument,      "create cache dir" },
-  { C_X, OPT_LOG_DIR,     '\0', "log-dir",   no_argument,      "create log dir" },
-  { C_X, OPT_LOGIN,       '\0', "login",     no_argument,      "simulate login environment" },
+  { C_X, OPT_RUN_DIR,     '\0', "run-dir",   no_argument,      "create run dir", NULL },
+  { C_X, OPT_STATE_DIR,   '\0', "state-dir", no_argument,      "create state dir", NULL },
+  { C_X, OPT_CACHE_DIR,   '\0', "cache-dir", no_argument,      "create cache dir", NULL },
+  { C_X, OPT_LOG_DIR,     '\0', "log-dir",   no_argument,      "create log dir", NULL },
+  { C_X, OPT_LOGIN,       '\0', "login",     no_argument,      "simulate login environment", NULL },
   { C_X, OPT_OOM,         '\0', "oom",       required_argument,"set oom adjust value", "ADJ" },
+  { C_X, OPT_HARDLIMIT,   '\0', "hardlimit", no_argument,      "set hard limits with soft limits", NULL },
 };
 #define max_options ((ssize_t) ((sizeof options_info / sizeof *options_info)))
 
@@ -275,6 +265,8 @@ bool parse_limits(struct limit *limit, const char *arg) {
   if (sep != NULL) {
     *sep++ = '\0';
     limit->hard_specified = parse_limit(&limit->limits.rlim_max, sep);
+  } else if (set(OPT_HARDLIMIT)) {
+    both = true;
   }
   if (arg != sep) {
     limit->soft_specified = parse_limit(&limit->limits.rlim_cur, arg);
@@ -418,8 +410,8 @@ int sched_policy_from_name(const char *name) {
   return SCHED_OTHER;
 }
 
-static const struct option_info *find_option(int by_code,
-                                             const char *by_name) {
+const struct option_info *find_option(int by_code,
+                                      const char *by_name) {
   const struct option_info *optdef;
   const struct option_info *incompatible_option = NULL;
   enum compat_level compat = opt.app->compat_level;
@@ -702,7 +694,9 @@ static void handle_option(enum compat_level *compat,
   case OPT_PRIVATE_RUN:
   case OPT_PRIVATE_TMP:
   case OPT_PROTECT_HOME:
+  case OPT_RO_HOME:
   case OPT_RO_SYS:
+  case OPT_RO_ETC:
   case OPT_PGRPHACK:
   case OPT_FORK_JOIN:
   case OPT_NEW_ROOT:
@@ -712,6 +706,7 @@ static void handle_option(enum compat_level *compat,
   case OPT_CACHE_DIR:
   case OPT_LOG_DIR:
   case OPT_LOGIN:
+  case OPT_HARDLIMIT:
     /* Boolean options needing no further option processing */
     break;
   case OPT_CAPBS_KEEP:
@@ -840,6 +835,10 @@ static void handle_option(enum compat_level *compat,
     if (!parse_limits(&opt.rlimit_sigpending, optarg))
       opt.error = true;
     break;
+  case OPT_RLIMIT_LOCKS:
+    if (!parse_limits(&opt.rlimit_locks, optarg))
+      opt.error = true;
+    break;
   case OPT_CLOSE_STDIN:
   case OPT_CLOSE_STDOUT:
   case OPT_CLOSE_STDERR:
