@@ -82,6 +82,7 @@ enum opt /* C23: :int */ {
   OPT_RLIMIT_RTPRIO,
   OPT_RLIMIT_RTTIME,
   OPT_RLIMIT_SIGPENDING,
+  OPT_RLIMIT_LOCKS,
   OPT_CLOSE_STDIN,
   OPT_CLOSE_STDOUT = OPT_CLOSE_STDIN + STDOUT_FILENO - STDIN_FILENO,
   OPT_CLOSE_STDERR = OPT_CLOSE_STDIN + STDERR_FILENO - STDIN_FILENO,
@@ -100,7 +101,9 @@ enum opt /* C23: :int */ {
   OPT_PRIVATE_RUN,
   OPT_PRIVATE_TMP,
   OPT_PROTECT_HOME,
+  OPT_RO_HOME,
   OPT_RO_SYS,
+  OPT_RO_ETC,
   OPT_CAPBS_KEEP,
   OPT_CAPBS_DROP,
   OPT_CAPS_KEEP,
@@ -119,6 +122,7 @@ enum opt /* C23: :int */ {
   OPT_LOG_DIR,
   OPT_LOGIN,
   OPT_OOM,
+  OPT_HARDLIMIT,
 
   /* Keep at end */
   OPT_EXIT,
@@ -216,6 +220,7 @@ struct options {
   struct limit rlimit_rtprio;
   struct limit rlimit_rttime;
   struct limit rlimit_sigpending;
+  struct limit rlimit_locks;
   unsigned close_fds;
   enum cap_op cap_bounds_op;
   cap_bits_t cap_bounds;
@@ -256,5 +261,7 @@ void options_print_positional(FILE *out);
 void options_explain_positional(FILE *out);
 int options_parse(int argc, char *argv[]);
 void options_free(void);
+const struct option_info *find_option(int by_code,
+                                      const char *by_name);
 
 #endif
